<?php

/**
 * pomoc actions.
 *
 * @package    sam3
 * @subpackage pomoc
 * @author     Robert Górecki
 * @version    SVN: $Id: actions.class.php 12479 2008-10-31 10:54:40Z fabien $
 */
class rejestracjaActions extends myActions
{

    public function preExecute()
    {
        if(!in_array('regp', ModulPeer::getEnabled()))
        {
            $this->forward404("Moduł Rezerwacji nie jest włączony");
        }
    }

    /**
     * akcja widoku listy rejestracji osob
     * @author Tomasz Glowinski, 2012
     * @param sfRequest $request A request object
     */
    public function executeIndex(sfWebRequest $request)
    {
        // przechwycenie wymaganych parametrow
        $this->point_id = $request->getParameter('point_id');
        $point = RegistrationPointTblPeer::retrieveByPK($this->point_id);
        $this->forward404Unless($point, '[rejestracja/index] Nie ma takiego wydzialu ('.$this->point_id.')');

        // przechwycenie ewentualnych parametrow
        $week_number = $request->getParameter('wn');
        if(empty($week_number) || !is_numeric($week_number))
            $week_number = date('W');
        $year = $request->getParameter('year');
        if(empty($year) || !is_numeric($year))
            $year = date('Y');

        // odczyt zapisanych ustawien
        $prefix = 'registatrion_conf';
        $excluded_conf = UstawieniePeer::getProperties($prefix); 
        $this->excluded_conf = array();
        foreach($excluded_conf as $key => $val)
        {
            list($point_id, $dow, $hour, $nr) = explode('_', $key);
            
            if(!array_key_exists($point_id, $this->excluded_conf))
            {
                $this->excluded_conf[$point_id] = array();
            }
            
            if(!array_key_exists($dow, $this->excluded_conf[$point_id]))
            {
                $this->excluded_conf[$point_id][$dow] = array();
            }
            
            if(!array_key_exists($hour, $this->excluded_conf[$point_id][$dow]))
            {
                $this->excluded_conf[$point_id][$dow][$hour] = array();
            }
            
            $this->excluded_conf[$point_id][$dow][$hour][] = $val;
            
        }
        //devdump($this->excluded_conf, '', 0);
        
        // $config = UstawieniePeer::getProperties('regp_config'); // v1
        $config = $point->toArray(BasePeer::TYPE_FIELDNAME); // v2
        // wyliczenie najwczesniejszej i najpozniejszej godziny pracy
        $min_hour = 23;
        $min_minute = 59;
        $max_hour = 0;
        $max_minute = 0;
        $dow = array('mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun');
        foreach($dow as $d)
        {
            $day_cfg_min = $config[$d.'_start'];
            $day_cfg_max = $config[$d.'_stop'];
            if(!empty($day_cfg_min))
            {
                // akutalizacja najwczesniejszej godziny
                list($hour, $minute) = explode(':', $day_cfg_min);
                if($hour <= $min_hour)
                    if($hour < $min_hour || $minute < $min_minute)
                    {
                        $min_hour = $hour;
                        $min_minute = $minute;
                    }
            }
            if(!empty($day_cfg_max))
            {
                // akutalizacja najpozniejszej godziny
                list($hour, $minute) = explode(':', $day_cfg_max);
                if($hour >= $max_hour)
                    if($hour > $max_hour || $minute > $max_minute)
                    {
                        $max_hour = $hour;
                        $max_minute = $minute;
                    }
            }
        }
        // wyliczenie daty poniedzialku we wskazanym tygodniu
        $monday = new DateTime();
        $monday->setISODate($year, $week_number);

        // wyliczanie dat dni wskazanego tygodnia 
        $monday_str = $monday->format('Y-m-d');
        $week_dates = array();
        for($i = 0; $i < 7; $i++)
        {
            $day = new DateTime();
            $day->setDate($monday->format('Y'), $monday->format('m'), (int) $monday->format('d') + $i);
            $week_dates[$dow[$i]] = $day;
        }

        // wyliczenie nastepengo i poprzedniego tygodnia 
        $current_week = new DateTime();
        $current_week->setDate($monday->format('Y'), $monday->format('m'), (int) $monday->format('d') + 6);
        $next_week = new DateTime();
        $next_week->setDate($monday->format('Y'), $monday->format('m'), (int) $monday->format('d') + 7 + 6);
        $prev_week = new DateTime();
        $prev_week->setDate($monday->format('Y'), $monday->format('m'), (int) $monday->format('d') - 7 + 6);
        //print_r($next_week->format('W Y-m-d'));
        // wyliczenie czasu rozpoczecia i konca pracy dla kazdego dnia
        $start_time = array();
        $stop_time = array();
        foreach($dow as $d)
        {
            $start = explode(':', $config[$d.'_start']);
            if(count($start) > 1)
                $start_time[$d] = mktime($start[0], $start[1]);
            else
                $start_time[$d] = mktime(23, 59);
            $stop = explode(':', $config[$d.'_stop']);
            if(count($stop) > 1)
                $stop_time[$d] = mktime($stop[0], $stop[1]);
            else
                $stop_time[$d] = mktime(0, 0);
        }

        // wydobycie osob zarejestrowanych na ten tydzien
        $reg_list = RegistrationTblPeer::getRegistratedPersonsByWeek($week_number, $point->getId());
        $registered = array();
        foreach($reg_list as $regp)
        {
            $date = $regp['DATE'];
            if(array_key_exists($date, $registered))
                $registered[$date][$regp['ID']] = $regp;
            else
                $registered[$date] = array($regp['ID'] => $regp);
        }

        // lista dni wolnych od pracy
        $excluded_dates = explode(', ', trim($config['excluded_days']));
        
        
        // przekazanie zmiennycyh do widoku
        $this->min_hour = $min_hour;
        $this->min_minute = $min_minute;
        $this->max_time = mktime($max_hour, $max_minute);
        $this->config = $config;
        $this->week_dates = $week_dates;
        $this->current_weeks_number = $current_week->format('W');
        $this->current_weeks_year = $current_week->format('Y');
        $this->next_weeks_number = $next_week->format('W');
        $this->next_weeks_year = $next_week->format('Y');
        $this->prev_weeks_number = $prev_week->format('W');
        $this->prev_weeks_year = $prev_week->format('Y');
        $this->start_time = $start_time;
        $this->stop_time = $stop_time;
        $this->reg_list = $registered;
        $this->excluded_dates = $excluded_dates;
        $this->dow = $dow;
        $this->week_number = $week_number;
        $this->year = $year;
//        $this->extra_terms_conf = UstawieniePeer::getProperties('registration_conf_extra_terms_'.$point_id); 
//        devdump($this->extra_terms_conf, '', 0);
    }

    /**
     * akcja widoku konfiguracji rejestracji osob (oblsuguje takze zapis)
     * @author        Tomasz Glowinski, 2012
     * @param sfRequest $request A request object
     */
    public function executeConfig(sfWebRequest $request)
    {
        // obsluga zapisu formularza
        if($request->isMethod('post'))
        {
            try
            {
                // przechwycenie zgloszonych danych
                $config = $request->getParameter('regp');

                // walidacja interwalu
                if(empty($config['interval']) || !is_numeric($config['interval']))
                    throw new Exception('Wypełnij poprawnie wszystkie wymagane pola');

                //$regexp = "/^(0[1-9]|1[0-9]|2[0-3])\:([0-5][0-9])$/";
                $regexp = "/^((0?[0-9]|1[0-9]|2[0-3]):([0-5][0-9]))$/";
                // walidacja godzin - poniedzialek
                if(!empty($config['mon_start']) || !empty($config['mon_stop']))
                    if(preg_match($regexp, $config['mon_start']) != 1 || preg_match($regexp, $config['mon_stop']) != 1)
                        throw new Exception('Nie określono poprawnie zakresu czasu pracy');
                // walidacja godzin - wtorek
                if(!empty($config['tue_start']) || !empty($config['tue_stop']))
                    if(preg_match($regexp, $config['tue_start']) != 1 || preg_match($regexp, $config['tue_stop']) != 1)
                        throw new Exception('Nie określono poprawnie zakresu czasu pracy');
                // walidacja godzin - sroda
                if(!empty($config['wed_start']) || !empty($config['wed_stop']))
                    if(preg_match($regexp, $config['wed_start']) != 1 || preg_match($regexp, $config['wed_stop']) != 1)
                        throw new Exception('Nie określono poprawnie zakresu czasu pracy');
                // walidacja godzin - czwartek
                if(!empty($config['thu_start']) || !empty($config['thu_stop']))
                    if(preg_match($regexp, $config['thu_start']) != 1 || preg_match($regexp, $config['thu_stop']) != 1)
                        throw new Exception('Nie określono poprawnie zakresu czasu pracy');
                // walidacja godzin - piatek
                if(!empty($config['fri_start']) || !empty($config['fri_stop']))
                    if(preg_match($regexp, $config['fri_start']) != 1 || preg_match($regexp, $config['fri_stop']) != 1)
                        throw new Exception('Nie określono poprawnie zakresu czasu pracy');

                // walidacja dat wykluczonych
                $regexp = "/^( ?[12][09][0-9][0-9]\-[01][0-9]\-[0-3][0-9])+$/";
                if(array_key_exists('excluded', $config) && !empty($config['excluded']))
                    if(preg_match($regexp, $config['excluded']) != 1)
                        throw new Exception('Nie określono poprawnie dni wolnych od pracy');

                // walidacja adresu email
                $regexp = '/^[A-Za-z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4}$/';
                if(array_key_exists('notice_email', $config) && !empty($config['notice_email']))
                    if(preg_match($regexp, $config['notice_email']) != 1)
                        throw new Exception('Nie określono poprawnie adresu email do wysyłki powiadomienia');

                // zapisanie ustawien 
                UstawieniePeer::setProperties('regp_config', $config);

                // komunikat o sukcesie (domniemanym)
                $this->getUser()->setFlash('notice', 'Zapisano ustawienia.');
            }
            catch (Exception $e)
            {
                $this->getUser()->setFlash('error', $e->getMessage());
            }

            // odswiezenie strony
            $this->redirect('rejestracja/config');
        }

        // odczytanie zapisanej konfiguracji
        $config = UstawieniePeer::getProperties('regp_config');
        if(array_key_exists('interval', $config))
            $this->interval = $config['interval'];
        else
            $this->interval = null;
        if(array_key_exists('mon_start', $config))
            $this->mon_start = $config['mon_start'];
        else
            $this->mon_start = null;
        if(array_key_exists('mon_stop', $config))
            $this->mon_stop = $config['mon_stop'];
        else
            $this->mon_stop = null;
        if(array_key_exists('tue_start', $config))
            $this->tue_start = $config['tue_start'];
        else
            $this->tue_start = null;
        if(array_key_exists('tue_stop', $config))
            $this->tue_stop = $config['tue_stop'];
        else
            $this->tue_stop = null;
        if(array_key_exists('wed_start', $config))
            $this->wed_start = $config['wed_start'];
        else
            $this->wed_start = null;
        if(array_key_exists('wed_stop', $config))
            $this->wed_stop = $config['wed_stop'];
        else
            $this->wed_stop = null;
        if(array_key_exists('thu_start', $config))
            $this->thu_start = $config['thu_start'];
        else
            $this->thu_start = null;
        if(array_key_exists('thu_stop', $config))
            $this->thu_stop = $config['thu_stop'];
        else
            $this->thu_stop = null;
        if(array_key_exists('fri_start', $config))
            $this->fri_start = $config['fri_start'];
        else
            $this->fri_start = null;
        if(array_key_exists('fri_stop', $config))
            $this->fri_stop = $config['fri_stop'];
        else
            $this->fri_stop = null;
        if(array_key_exists('sat_start', $config))
            $this->sat_start = $config['sat_start'];
        else
            $this->sat_start = null;
        if(array_key_exists('sat_stop', $config))
            $this->sat_stop = $config['sat_stop'];
        else
            $this->sat_stop = null;
        if(array_key_exists('sun_start', $config))
            $this->sun_start = $config['sun_start'];
        else
            $this->sun_start = null;
        if(array_key_exists('sun_stop', $config))
            $this->sun_stop = $config['sun_stop'];
        else
            $this->sun_stop = null;
        if(array_key_exists('excluded', $config))
            $this->excluded = $config['excluded'];
        else
            $this->excluded = null;
        if(array_key_exists('notice_email', $config))
            $this->notice_email = $config['notice_email'];
        else
            $this->notice_email = null;
        if(array_key_exists('reg_msg', $config))
            $this->reg_msg = $config['reg_msg'];
        else
            $this->reg_msg = null;
        if(array_key_exists('cfm_msg', $config))
            $this->cfm_msg = $config['cfm_msg'];
        else
            $this->cfm_msg = null;
        if(array_key_exists('eml_msg', $config))
            $this->eml_msg = $config['eml_msg'];
        else
            $this->eml_msg = null;

        $url = sfContext::getInstance()->getRequest()->getRelativeUrlRoot();
        $this->reg_msg_widget = new sfWidgetFormTextareaTinyMCE(
            array(
            'config' => '
                language: "pl",
                width: "455",
                content_css : "'.$url.'/css/tinyStyle.css",
                theme : "advanced",
                plugins : "advimage,advlink,media,contextmenu",
                plugins : "ankiety,safari,pagebreak,style,layer,table,save,advhr,advimage,advlink,emotions,iespell,inlinepopups,insertdatetime,media,searchreplace,print,contextmenu,paste,directionality,fullscreen,noneditable,visualchars,nonbreaking,xhtmlxtras,template",
                theme_advanced_buttons1 : "bold,italic,underline,justifyleft,justifycenter,justifyright,justifyfull,link,unlink",
                theme_advanced_buttons2 : "",
                theme_advanced_buttons3 : "",
                theme_advanced_buttons4 : "",
                theme_advanced_buttons5 : "",
                theme_advanced_toolbar_location : "top",
                theme_advanced_toolbar_align : "left",
                theme_advanced_statusbar_location : "bottom",
                theme_advanced_resizing : true,
                file_browser_callback : "ajaxfilemanager"'), array('class' => 'tiny_mce_pl1'));

        $this->cfm_msg_widget = new sfWidgetFormTextareaTinyMCE(
            array(
            'config' => '
                language: "pl",
                width: "455",
                content_css : "'.$url.'/css/tinyStyle.css",
                theme : "advanced",
                plugins : "advimage,advlink,media,contextmenu",
                plugins : "ankiety,safari,pagebreak,style,layer,table,save,advhr,advimage,advlink,emotions,iespell,inlinepopups,insertdatetime,media,searchreplace,print,contextmenu,paste,directionality,fullscreen,noneditable,visualchars,nonbreaking,xhtmlxtras,template",
                theme_advanced_buttons1 : "bold,italic,underline,justifyleft,justifycenter,justifyright,justifyfull,link,unlink",
                theme_advanced_buttons2 : "",
                theme_advanced_buttons3 : "",
                theme_advanced_buttons4 : "",
                theme_advanced_buttons5 : "",
                theme_advanced_toolbar_location : "top",
                theme_advanced_toolbar_align : "left",
                theme_advanced_statusbar_location : "bottom",
                theme_advanced_resizing : true,
                file_browser_callback : "ajaxfilemanager"'), array('class' => 'tiny_mce_pl2'));

        $this->eml_msg_widget = new sfWidgetFormTextareaTinyMCE(
            array(
            'config' => '
                language: "pl",
                width: "455",
                content_css : "'.$url.'/css/tinyStyle.css",
                theme : "advanced",
                plugins : "advimage,advlink,media,contextmenu",
                plugins : "ankiety,safari,pagebreak,style,layer,table,save,advhr,advimage,advlink,emotions,iespell,inlinepopups,insertdatetime,media,searchreplace,print,contextmenu,paste,directionality,fullscreen,noneditable,visualchars,nonbreaking,xhtmlxtras,template",
                theme_advanced_buttons1 : "bold,italic,underline,justifyleft,justifycenter,justifyright,justifyfull,link,unlink",
                theme_advanced_buttons2 : "",
                theme_advanced_buttons3 : "",
                theme_advanced_buttons4 : "",
                theme_advanced_buttons5 : "",
                theme_advanced_toolbar_location : "top",
                theme_advanced_toolbar_align : "left",
                theme_advanced_statusbar_location : "bottom",
                theme_advanced_resizing : true,
                file_browser_callback : "ajaxfilemanager"'), array('class' => 'tiny_mce_pl3'));
    }

    /**
     * akcja dodawania nowej rejestracji 
     * @author        Tomasz Glowinski, 2012
     * @param sfRequest $request A request object
     */
    public function executeNew(sfWebRequest $request)
    {
        $point_id = $request->getParameter('point_id');
        $this->point_id = $point_id;
        if(empty($point_id))
        {
            $this->forward404('[rejestracja/new] Nie ma takiego wydzialu');
        }

        if($request->isMethod('post'))
        {
            // przechwycenie zgloszonych danych
            $date = $request->getParameter('date');
            $time = urldecode($request->getParameter('time'));

            // walidacja danych
            if(empty($date) || empty($time))
            {
                $this->getUser()->setFlash('error', 'Niepoprawny parametr');
                $this->redirect('rejestracja/index?point_id='.$point_id);
            }

            // obiekt formy    
            $this->form = new RegistrationTblForm();
            $this->form->setDefault('date', $date.' '.$time);
            $this->form->setDefault('status', 0);
            $this->form->setDefault('tel', '');
            $this->form->setDefault('point_id', $point_id);
        }
        elseif($request->isMethod('get'))
        {
            // przechwycenie zgloszonych danych
            $params = $request->getParameter('params');

            if(empty($params))
            {
                $this->getUser()->setFlash('error', 'Niedozwolona operacja');
                $this->redirect('rejestracja/index');
            }

            // obiekt formy    
            $this->form = new RegistrationTblForm();
            $this->form->bind(unserialize($params));
        }
        else
        {
            $this->getUser()->setFlash('error', 'Niedozwolona operacja');
            $this->redirect('rejestracja/index');
        }
    }

    /**
     * akcja walidujaca dene i rejestrujaca osobe 
     * @param sfWebRequest $request 
     */
    public function executeRegister(sfWebRequest $request)
    {
        $params = $request->getParameter('registration_tbl');
        unset($params['id']);

        try
        {
            // walidacja zgloszenia
            if(!$request->isMethod('post'))
                throw new Exception('Niedozwolona operacja');

            // nowy obiekt formy
            $this->form = new RegistrationTblForm();

            // przechwycenie i oczyszczenie zgloszonych danych            
            $params['status'] = 1;
            $safe_params = Functions::cleanValuesFromForm($this->form, $params);

            $this->point_id = $params['point_id'];
            $point = RegistrationPointTblPeer::retrieveByPK($this->point_id);

            // laczenie danych z forma
            $this->form->bind($safe_params);

            // walidacja formy
            if(!($this->form->isValid()))
                throw new Exception('Wypełnij poprawnie wszystkie wymagane pola');

            // weryfikacja dostepnosci terminu
            if(RegistrationTblPeer::checkAvailabilityByDate($params['date'].':00', $this->point_id) == false)
                throw new Exception('Termin jest już zajęty');

            // aktualizacja statystyk
            StatisticsTblPeer::increaseUses('regp', $this->point_id);

            // zapisanie rejestracji
            $this->form->save();

            // komunikat o powodzeniu
            $this->getUser()->setFlash('notice', 'Zapisano rejestrację');

            // przekierowanie na widok listy
            $this->redirect('rejestracja/index?point_id='.$this->point_id);
        }
        catch (sfStopException $e)
        {
            exit;
        }
        catch (Exception $e)
        {
            // komunikat o bledzie
            $this->getUser()->setFlash('error', $e->getMessage());

            // przekierowanie na widok listy
            $this->redirect('rejestracja/new?point_id='.$params['point_id'].'&params='.serialize($params));
        }
    }

    /**
     * akcja usuwajaca rejestracje
     * @param sfWebRequest $request 
     */
    public function executeUsun(sfWebRequest $request)
    {
        $id = $request->getParameter('id');
        $reg = RegistrationTblPeer::retrieveByPK($id);

        if(empty($reg))
        {
            $this->getUser()->setFlash('error', 'Nie ma takiej rezerwacji ('.$id.').');
            $this->redirect('@homepage');
        }

        $point_id = $reg->getPointId();

        // aktualizacja statystyk        
        try
        {
            if($reg->getStatus() == RegistrationTblPeer::$STATUS_CONFIRMED)
            {
                StatisticsTblPeer::decreaseUses('regp', $point_id);
            }
        }
        catch (Exception $e)
        {
            $this->getUser()->setFlash('error', 'Nie udało się zaktualizować statystyk.');
        }

        $reg->delete();
        $this->getUser()->setFlash('notice', 'Rezerwacja usunięta.');
        $this->redirect('rejestracja/index?point_id='.$point_id);
    }

    /**
     * akcja blokujaca termin
     * @param sfWebRequest $request 
     */
    public function executeBlock(sfWebRequest $request)
    {
        $point_id = $request->getParameter('point_id');

        try
        {
            // walidacja zgloszenia
            if(!$request->isMethod('post'))
                throw new Exception('Niedozwolona operacja');

            // nowy obiekt formy
            $this->form = new RegistrationTblForm();

            $safe_params = array(
                'point_id' => $point_id,
                'date' => $request->getParameter('date').' '.$request->getParameter('time'),
                'name' => 'blocked',
                'last_name' => 'blocked',
                'tel' => '0',
                'email' => 'mailer@sam3.pl',
                'status' => 8,
            );

            // laczenie danych z forma
            $this->form->bind($safe_params);

            // walidacja formy
            if(!($this->form->isValid()))
                throw new Exception('Wypełnij poprawnie wszystkie wymagane pola');

            // weryfikacja dostepnosci terminu
            if(RegistrationTblPeer::checkAvailabilityByDate($params['date'].':00', $point_id) == false)
                throw new Exception('Termin jest już zajęty');

            // zapisanie rejestracji
            $result = $this->form->save();

            // komunikat o powodzeniu
            $this->getUser()->setFlash('notice', 'Zablokowano termin');

            // przekierowanie na widok listy
            $this->redirect('rejestracja/index?point_id='.$point_id);
        }
        catch (sfStopException $e)
        {
            exit;
        }
        catch (Exception $e)
        {
            // komunikat o bledzie
            $this->getUser()->setFlash('error', $e->getMessage());

            // przekierowanie na widok listy
            $this->redirect('rejestracja/index?point_id='.$point_id);
        }
    }

    public function executeRaport(sfWebRequest $request)
    {
        try
        {
            $params = $request->getParameterHolder()->getAll();
            $point_id = array_key_exists('point_id', $params) ? $params['point_id'] : null;
            $point = RegistrationPointTblPeer::retrieveByPK($point_id);
            if(empty($point))
            {
                throw new Exception('Nie ma takiego wydziału ('.$point_id.').');
            }

            // odczyt zapisanych ustawien
            // $config = UstawieniePeer::getProperties('regp_config'); // v1
            $config = $point->toArray(BasePeer::TYPE_FIELDNAME); // v2
            // wybrany tydzien
            $week = array_key_exists('wn', $params) ? $params['wn'] : date("W");
            $year = array_key_exists('year', $params) ? $params['year'] : date('Y');
            $chosen_date = date('Y-m-d', strtotime(array_key_exists('date', $params) ? $params['date'] : date('d-m-Y')));
            
            // lista dni wolnych od pracy
            $excluded_dates = explode(', ', trim($config['excluded_days']));

            // lista zapisanych osob w tym tygodniu
            $registrated = RegistrationTblPeer::getRegistratedPersonsByWeek($week, $point_id);
            if(empty($registrated))
            {
                throw new Exception('Lista jest pusta.');
            }
            $reg_list = array();
            foreach($registrated as $regp)
            {
                $date = $regp['DATE'];
                if(array_key_exists($date, $reg_list))
                    $reg_list[$date][$regp['ID']] = $regp;
                else
                    $reg_list[$date] = array($regp['ID'] => $regp);
            }
            //devdump($reg_list, '', 0);
            // wyliczenie najwczesniejszej i najpozniejszej godziny pracy
            $min_hour = 23;
            $min_minute = 59;
            $max_hour = 0;
            $max_minute = 0;
            $dow = array('mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun');
            foreach($dow as $d)
            {
                $day_cfg_min = $config[$d.'_start'];
                $day_cfg_max = $config[$d.'_stop'];
                if(!empty($day_cfg_min))
                {
                    // akutalizacja najwczesniejszej godziny
                    list($hour, $minute) = explode(':', $day_cfg_min);
                    if($hour <= $min_hour)
                        if($hour < $min_hour || $minute < $min_minute)
                        {
                            $min_hour = $hour;
                            $min_minute = $minute;
                        }
                }
                if(!empty($day_cfg_max))
                {
                    // akutalizacja najpozniejszej godziny
                    list($hour, $minute) = explode(':', $day_cfg_max);
                    if($hour >= $max_hour)
                        if($hour > $max_hour || $minute > $max_minute)
                        {
                            $max_hour = $hour;
                            $max_minute = $minute;
                        }
                }
            }
            // wyliczenie daty poniedzialku we wskazanym tygodniu
            $monday = new DateTime();
            $monday->setISODate($year, $week);

            // wyliczanie dat dni wskazanego tygodnia 
            $monday_str = $monday->format('Y-m-d');
            $week_dates = array();
            for($i = 0; $i < 7; $i++)
            {
                $day = new DateTime();
                $day->setDate($monday->format('Y'), $monday->format('m'), (int) $monday->format('d') + $i);
                $week_dates[$dow[$i]] = $day;
            }

            // wyliczenie czasu rozpoczecia i konca pracy dla kazdego dnia
            $start_time = array();
            $stop_time = array();
            foreach($dow as $d)
            {
                $start = explode(':', $config[$d.'_start']);
                if(count($start) > 1)
                    $start_time[$d] = mktime($start[0], $start[1]);
                else
                    $start_time[$d] = mktime(23, 59);
                $stop = explode(':', $config[$d.'_stop']);
                if(count($stop) > 1)
                    $stop_time[$d] = mktime($stop[0], $stop[1]);
                else
                    $stop_time[$d] = mktime(0, 0);
            }


            // -- PDF -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -
            $config_pdf = sfTCPDFPluginConfigHandler::loadConfig();

            // pdf object
            $pdf = new sfTCPDF();

            // settings
//            $naglowek = 'Wygenerowano: '.date('Y-m-d H:i:s').' przez '.$this->getUser()->getGuardUser()->getFullName();
            $naglowek = 'Rejestr zapisanych osób '.date('j. m Y', strtotime($chosen_date));

            $pdf->SetFont("FreeSerif", "", 10);
            $pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
            $pdf->setHeaderFont(array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));
            $pdf->SetHeaderData(PDF_HEADER_LOGO, PDF_HEADER_LOGO_WIDTH, PDF_HEADER_TITLE, $naglowek);
            $pdf->setFooterFont(array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));
            $pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
            $pdf->SetFooterMargin(PDF_MARGIN_FOOTER);
            $border = array('RB' => array('width' => 0.1, 'cap' => 'butt', 'join' => 'miter', 'dash' => 0, 'color' => array(127, 127, 127)));

            // init pdf doc
            $pdf->AliasNbPages();
            $pdf->AddPage();

            $pdf->SetFont("FreeSerif", "B", 16);
            $pdf->SetCellPadding(1);
            $pdf->Cell(190, 0, $point->getNazwa(), '', $ln = true, $align = 'L', $fill = false);
            $pdf->SetFont("FreeSerif", "", 10);
            $pdf->Cell(25, 0, 'Wygenerowano:', '', $ln = false, $align = 'R', $fill = false);
            $pdf->Cell(165, 0, date('j. m Y, H:i'), '', $ln = true, $align = 'L', $fill = false);
            //$pdf->Cell(30, 0, '', '', $ln = false, $align = 'R', $fill = false);
            //$pdf->Cell(160, 0, $this->getUser()->getGuardUser() ? $this->getUser()->getGuardUser()->getFullName() : 'niezalogowany użytkownik', '', $ln = true, $align = 'L', $fill = false);
            //$pdf->Cell(30, 0, 'Ilość stanowisk:', '', $ln = false, $align = 'R', $fill = false);
            //$pdf->Cell(160, 0, $point->getParallelPoints(), '', $ln = true, $align = 'L', $fill = false);
            //$pdf->Cell(30, 0, 'Czas rejestracji:', '', $ln = false, $align = 'R', $fill = false);
            //$pdf->Cell(160, 0, $point->getInterval().' minut', '', $ln = true, $align = 'L', $fill = false);
            //$pdf->Ln();
            //$pdf->Ln();

            $this->addPdfHeader($pdf, $week_dates, $dow, $chosen_date);

            $minute = $min_minute;
            $time = mktime($min_hour, $minute);
            $iter = 0;
            $max_time = mktime($max_hour, $max_minute);
            $lp = 1;
            while($time < $max_time)
            {
                $row = array();
                $row_heights = array();

                $row[0] = date('G:i', $time);
                $row_heights[0] = $pdf->getNumLines(date('G:i', $time), 15);

                for($i = 0; $i < 7; $i++)
                {
                    $date = $week_dates[$dow[$i]]->format('Y-m-d');

                    if($date != $chosen_date)
                    {
                        continue;
                    }

                    $term = $date.date(' H:i:00', $time);
                    $row[$i + 1] = array();
                    $row_heights[$i + 1] = 1;

                    // jesli dzien nie jest wolny od pracy a ta godzina miesci sie w zakresie czasu pracy tego dnia
                    if(!in_array(date('d-m-Y', strtotime($date)), $excluded_dates) && ($start_time[$dow[$i]] <= $time) && ($stop_time[$dow[$i]] > $time))
                    {
                        $conf = UstawieniePeer::getProperties('registatrion_conf_'.$point_id.'_'.$i.'_'.date('H:i', $time));
                        
                        if(array_key_exists($term, $reg_list))
                        {
                            $max_h = 0;
                            $y = 0;
                            foreach($reg_list[$term] as $regp)
                            {
                                $txt = $lp.'. '.$regp['NAME'].' '.$regp['LAST_NAME'].' ('.date('d-m-Y H:i', strtotime($regp['CREATED_AT'])).')';
                                if($regp['STATUS'] & 1)
                                {
                                    $row[$i + 1][] = $txt;
                                    $max_h += $pdf->getNumLines($txt, 175);
                                    $y++;
                                    $lp++;
                                }
                                elseif($regp['STATUS'] & 2 || $regp['STATUS'] == 0)
                                {
                                    //$row[$i+1][] = '<span style="text-decoration: line-through;">'.$txt.'</span>';
                                }
                                elseif($regp['STATUS'] & 8)
                                {
                                    $y++; 
                                }
                            }

                            // uzupelniam pozycje do pelna
                            for($y; $y < ($config['parallel_points'] - count($conf)); $y++)
                            {
                                $txt = $lp++.'. '.PHP_EOL;
                                $row[$i + 1][] = $txt;
                                $max_h += 2;
                            }

                            $row_heights[$i + 1] = $max_h;
                        }
                        else
                        {
                            // calkowicie wypelniam pustymi liniami
                            for($y = 0; $y < ($config['parallel_points'] - count($conf)); $y++)
                            {
                                $txt = $lp++.'. '.PHP_EOL;
                                $row[$i + 1][] = $txt;
                                $row_heights[$i + 1] += 2;
                            }
                        }

                        // dodaje odpowiednia ilosc pustych linii z powodu `extra_terms`
                        $extra_terms_conf = UstawieniePeer::getProperties('registration_conf_extra_terms_'.$point_id.'_'.$i.'_'.date('H:i', $time));
                        $extra_terms = intval(count($extra_terms_conf) > 0 ? array_shift($extra_terms_conf) : $config['extra_terms']);                        
                        if($extra_terms > 0)
                        {
                            for($x = 0; $x < $extra_terms; $x++)
                            {
                                $txt = $lp++.'. '.PHP_EOL;
                                $row[$i + 1][] = $txt;
                            }
                            $row_heights[$i + 1] += 2 * $extra_terms;
                        }

                        /* if(empty($row[$i+1]))
                          {
                          $row[$i+1][] = '';
                          $row_heights[$i+1] = 0;
                          } */
                    }
                    else
                    {
                        $row[$i + 1][] = '';
                        $row_heights[$i + 1] = 0;
                    }
                }
                
                $h = max(1, max($row_heights)) * 4.5;
                
                if($pdf->GetY() + $h > 277)
                {
                    $pdf->AddPage();
                    $this->addPdfHeader($pdf, $week_dates, $dow, $chosen_date);
                }

                // v1 - wersja do 13:04 2016-12-13
                /*foreach($row as $key => $val)
                {
                    $w = ($key == 0) ? 15 : 175;
                    $txt = is_array($val) ? implode(PHP_EOL, $val) : $val;
                    $pdf->MultiCell($w, $h, $txt, $key == 0 && $txt ? '' : $border, $align = $key == 0 ? 'R' : 'L', $fill = $key == 0, $ln = false);
                }*/
                // v2:
                $row_head = array_shift($row);
                foreach(array_shift($row) as $key => $entry)
                {
                    $h2 = $pdf->getNumLines($entry, 175) * 4.5;
                    $pdf->MultiCell(15, $h2, $key == 0 ? $row_head : '', '', $align = 'R', $fill = true, $ln = false);
                    $pdf->MultiCell(165, $h2, $entry, $border, $align = 'L', $fill = false, $ln = false);
                    $pdf->Ln();
                }
                //devdump(max(1, max($row_heights)), 'height', 0);
                //devdump($row, '', 1);

                $minute += $config['interval'];
                $iter++;
                $time = mktime($min_hour, $minute);
            }
            $pdf->lastPage();
            
            // output
            $title = Functions::przygotujNazwe('Raport rejestracji '.$point->getNazwa().' z '.$chosen_date);
            $title .= ".pdf";

            // katalog raportow
            if(!is_dir(sfConfig::get('sf_web_dir').'/raporty/'))
                mkdir(sfConfig::get('sf_web_dir').'/raporty/');
            if(!is_dir(sfConfig::get('sf_web_dir').'/raporty/'))
                throw new Exception('Nie można utworzyć katalogu raportów');

            // utworzenie PDF-a raportu
            $usr = $this->getUser()->getGuardUser();
            if(empty($usr))
            {
                $pdf->Output(sfConfig::get('sf_web_dir').'/raporty/'.$title, 'F'); // zapis na serwer
                $rel_url = $request->getUriPrefix().$request->getRelativeUrlRoot().'/raporty/'.$title;
                $this->redirect($rel_url);
            }
            else
            {
                $pdf->Output($title, 'I'); // przekierowanie do przegladarki
            }

            exit;
        }
        catch (sfStopException $e)
        {
            
        }
        catch (Exception $e)
        {
            $this->getUser()->setFlash('error', $e->getMessage());
            $this->redirect($request->getReferer());
        }
    }

    private function addPdfHeader(sfTCPDF $pdf, $week_dates, $dow, $chosen_date)
    {
        $dow_names = array('Poniedziałek', 'Wtorek', 'Środa', 'Czwartek', 'Piątek', 'Sobota', 'Niedziela');
        for($i = 0; $i < 7; $i++)
        {
            $date = $week_dates[$dow[$i]]->format('Y-m-d');

            if($date != $chosen_date)
            {
                continue;
            }

            $pdf->SetFont("FreeSerif", "", 12);
            $pdf->SetFillColor(231, 231, 231);
            $pdf->Cell(15, 0, '', '', $ln = false, $align = 'L', $fill = true);
            $pdf->Cell(165, 0, $dow_names[$i], '', $ln = true, $align = 'L', $fill = true);
            $pdf->SetFont("FreeSerif", "B", 12);
            $pdf->Cell(15, 0, '', '', $ln = false, $align = 'L', $fill = true);
            $pdf->Cell(165, 0, date('j-m-Y', strtotime($date)), '', $ln = false, $align = 'L', $fill = true);
        }
        $pdf->Ln();
        $pdf->SetFont("FreeSerif", "", 10);
    }

    public function executeStatistics(sfWebRequest $request)
    {
        $this->date_from = str_replace('_', '-', $request->getParameter('date_from', date('Y-m-01')));
        $this->date_to = str_replace('_', '-', $request->getParameter('date_to', date('Y-m-t')));
        $this->wydzialy = RegistrationPointTblPeer::getActive();
        $this->stats = array();
        foreach($this->wydzialy as $wydzial)
        {
            $this->stats[$wydzial->getId()] = StatisticsTblPeer::getStats('regp', $this->date_from, $this->date_to, $wydzial->getId());
        }

        $this->ajax = $request->isXmlHttpRequest();
        if($this->ajax) // AJAX
        {
            $this->setLayout(false);
        }
    }

    public function executeSaveConf(sfWebRequest $request)
    {
        $this->setLayout(false);
        
        try
        {
            if(!$request->isMethod('post'))
            {
                throw new Exception('Dostęp ograniczony.');
            }
            
            $conf_json = $request->getParameter('conf', null);
            if(is_null($conf_json))
            {
                throw new Exception('Brak parametru - conf');
            }
            
            $conf = json_decode($conf_json);
            if(empty($conf))
            {
                throw new Exception('Nie wybrano żadnego pola');
            }
            
            $chosen_action = $request->getParameter('chosen_action', null);
            if(is_null($chosen_action))
            {
                throw new Exception('Brak parametru - chosen_action');
            }
            if(empty($chosen_action))
            {
                throw new Exception('Wybierz akcję');
            }
            
            if($chosen_action == 'wyklucz')
            {
                $conf_ar = $this->convertRegConf($conf);
                
                foreach($conf_ar as $prefix => $values)
                {
                    $cnf = UstawieniePeer::getProperties($prefix);
                    foreach($values as $val)
                    {
                        if(!in_array($val, $cnf))
                        {
                            $cnf[] = $val;
                        }
                    }
                    UstawieniePeer::setProperties($prefix, $cnf);
                }
                
                $mask = "../cache/objects/registatrion_conf*.cache";
                array_map( "unlink", glob( $mask ) );                
                
                echo '<span class="notice">'.('Wybrane pola zostały wykluczone ze schematu').'</span>';
                $this->getUser()->setFlash('notice', 'Wybrane pola zostały wykluczone ze schematu');
            }
            elseif($chosen_action == 'wlacz')
            {
                $conf_ar = $this->convertRegConf($conf);
                //devdump($conf_ar, '', 0);
                foreach($conf_ar as $prefix => $values)
                {
                    $prop = array_flip(UstawieniePeer::getProperties($prefix));
                    
                    foreach($values as $val)
                    {
                        if(array_key_exists($val, $prop))
                        {
                            unset($prop[$val]);
                        }
                    }
                    
                    UstawieniePeer::deleteProperties($prefix);
                    UstawieniePeer::setProperties($prefix, array_flip($prop));
                }
                
                $mask = "../cache/objects/registatrion_conf*.cache";
                array_map( "unlink", glob( $mask ) );
                
                echo '<span class="notice">'.('Wybrane pola zostały udostępnione').'</span>';
                $this->getUser()->setFlash('notice', 'Wybrane pola zostały udostępnione');
            }
            else
            {
                echo '<span class="error">'.('Nieznana akcja').'</span>';
                $this->getUser()->setFlash('error', 'Nieznana akcja');
            }
        }
        catch (Exception $e)
        {
            echo '<span class="error">'.$e->getMessage().'</span>';
            $this->getUser()->setFlash('error', $e->getMessage());
        }
        
        return sfView::NONE;
    }
    
    private function convertRegConf($conf)
    {
        $ret = array();
        foreach($conf as $row)
        {
            $matches = null;
            preg_match('`([a-zA-Z_-]+)\[([0-9]+)\]\[([0-9]+)\]\[([0-9:]+)\]\[([0-9]+)\]`', $row, $matches);
            
            if(count($matches) < 5)
            {
                continue;
            }
            
            list($regex, $prefix, $point_id, $dow, $hour, $nr) = $matches;
            
//            if(!array_key_exists($prefix, $ret))
//            {
//                $ret[$prefix] = array();
//            }
//            
//            if(!array_key_exists($point_id, $ret[$prefix]))
//            {
//                $ret[$prefix][$point_id] = array();
//            }
//            
//            if(!array_key_exists($dow, $ret[$prefix][$point_id]))
//            {
//                $ret[$prefix][$point_id][$dow] = array();
//            }
            
            if(!array_key_exists($prefix.'_'.$point_id.'_'.$dow.'_'.$hour, $ret))
            {
                $ret[$prefix.'_'.$point_id.'_'.$dow.'_'.$hour] = array();
            }
            
            $ret[$prefix.'_'.$point_id.'_'.$dow.'_'.$hour][] = $nr;
            
        }
        
        return $ret;
    }

    public function executeUpdateExtraTermsConf(sfWebRequest $request)
    {
        $this->setLayout(false);
        
        try
        {
            $point_id = $request->getParameter('point_id');
            if(empty($point_id))
            {
                throw new Exception('Nie wybrano wydziału');
            }
            $point = RegistrationPointTblPeer::retrieveByPK($point_id);
            if(empty($point))
            {
                throw new Exception('Nie ma takiego wydziału ('.$point_id.')');
            }
            
            $dow = $request->getParameter('dow', false);
            if($dow === false)
            {
                throw new Exception('Nie wybrano dnia tygodnia');
            }
            
            $term = $request->getParameter('term');
            if(empty($term))
            {
                throw new Exception('Nie wybrano dnia terminu');
            }
            
            $value = $request->getParameter('value', $point->getExtraTerms());
            if(is_null($value))
            {
                throw new Exception('Nie podano wartości');
            }
            
            // nazwa ustawienia
            $prefix = 'registration_conf_extra_terms_'.$point_id.'_'.$dow.'_'.$term;
            
            // zapis do bazy
            if($value == '')
            {
                UstawieniePeer::deleteProperties($prefix);
                //UstawieniePeer::setProperties($prefix, array($value));
                        
            }
            else
            {
                UstawieniePeer::setProperties($prefix, array($value));
            }
            
            // czyszczenie cache-u
            $mask = "../cache/objects/registatrion_conf*.cache";
            array_map( "unlink", glob( $mask ) ); 
        }
        catch(Exception $e)
        {
            $this->msg = $e->getMessage();
            return sfView::ERROR;
        }
    }
}