<?php

/**
 * Akcje modulu Rejestracji osob.
 * @author     Tomasz Glowiski, 2012
 */
class rejestracjaActions extends sfActions
{
    /**
     * akcja listy rejestracji
     * @param       sfWebRequest $request
     * @author      Tomasz Glowinski, 2016
     */
    public function executeIndex(sfWebRequest $request)
    {
        $this->comp_id = 'regp_index';
        $this->view_mode = $request->getCookie('view_mode_'.$this->comp_id, 'view_list'); // sfConfig::get('app_view_mode')

        $c = new Criteria();
        $c->add(RegistrationPointTblPeer::IS_PUBLICATED, 1);
        $this->pager = new sfPropelPager('RegistrationPointTbl', sfConfig::get('app_pages'));
        $this->pager->setPage($request->getParameter('page', 1));
        $this->pager->setCriteria($c);
        $this->pager->init();
    }
    
    /**
     * akcja widoku listy rejestracji osob
     * @author Tomasz Glowinski, 2012
     * @param sfRequest $request A request object
     */
    public function executeShow(sfWebRequest $request)
    {
        $this->point_id = $request->getParameter('point_id');
        $this->forward404Unless($this->point_id);
        $this->point = RegistrationPointTblPeer::retrieveByPK($this->point_id);
        $this->forward404Unless($this->point);
        
        // przechwycenie ewentualnych parametrow
        $week_number = $request->getParameter('week');
        if(empty($week_number) || !is_numeric($week_number))
            $week_number = date('W');
        $year = $request->getParameter('year');
        if(empty($year) || !is_numeric($year))
            $year = date('Y');
        
        // odczyt zapisanych ustawien
        // $config = UstawieniePeer::getProperties('regp_config'); // v1
        $config = $this->point->toArray(BasePeer::TYPE_FIELDNAME); // v2

        // wyliczenie najwczesniejszej i najpozniejszej godziny pracy
        $min_hour = 23;
        $min_minute = 59;
        $max_hour = 0;
        $max_minute = 0;
        $dow = array('mon', 'tue', 'wed', 'thu', 'fri', 'sat', 'sun');
        foreach($dow as $d)
        {
            $day_cfg_min = $config[$d.'_start'];
            $day_cfg_max = $config[$d.'_stop'];
            if(!empty($day_cfg_min))
            {
                // akutalizacja najwczesniejszej godziny
                list($hour, $minute) = explode(':', $day_cfg_min);
                if($hour <= $min_hour)
                    if($hour < $min_hour || $minute < $min_minute)
                    {
                        $min_hour = $hour;
                        $min_minute = $minute;
                    }
            }
            if(!empty($day_cfg_max))
            {
                // akutalizacja najpozniejszej godziny
                list($hour, $minute) = explode(':', $day_cfg_max);
                if($hour >= $max_hour)
                    if($hour > $max_hour || $minute > $max_minute)
                    {
                        $max_hour = $hour;
                        $max_minute = $minute;
                    }
            }
        }
        // wyliczenie daty poniedzialku we wskazanym tygodniu
        $monday = new DateTime();
        $monday->setISODate($year, $week_number);

        // wyliczanie dat dni wskazanego tygodnia 
        $monday_str = $monday->format('Y-m-d');
        $week_dates = array();
        for($i = 0; $i < 7; $i++)
        {
            $day = new DateTime();
            $day->setDate($monday->format('Y'), $monday->format('m'), (int) $monday->format('d') + $i);
            $week_dates[$dow[$i]] = $day;
        }

        // wyliczenie nastepengo i poprzedniego tygodnia 
        $current_week = new DateTime();
        $current_week->setDate($monday->format('Y'), $monday->format('m'), (int) $monday->format('d') + 6);
        $next_week = new DateTime();
        $next_week->setDate($monday->format('Y'), $monday->format('m'), (int) $monday->format('d') + 7 + 6);
        $prev_week = new DateTime();
        $prev_week->setDate($monday->format('Y'), $monday->format('m'), (int) $monday->format('d') - 7 + 6);
        //print_r($next_week->format('W Y-m-d'));
        
        // wyliczenie czasu rozpoczecia i konca pracy dla kazdego dnia
        $start_time = array();
        $stop_time = array();
        foreach($dow as $d)
        {
            $start = explode(':', $config[$d.'_start']);
            if(count($start) > 1)
                $start_time[$d] = mktime($start[0], $start[1]);
            else
                $start_time[$d] = mktime(23, 59);
            $stop = explode(':', $config[$d.'_stop']);
            if(count($stop) > 1)
                $stop_time[$d] = mktime($stop[0], $stop[1]);
            else
                $stop_time[$d] = mktime(0, 0);
        }

        // wydobycie osob zarejestrowanych na ten tydzien
        $reg_list = RegistrationTblPeer::getShortRegistratedPersonsByWeek($week_number, $this->point_id);
        $registered = array();
        foreach($reg_list as $regp)
        {
            $date = $regp['DATE'];
            if(array_key_exists($date, $registered))
                $registered[$date][$regp['ID']] = $regp;
            else
                $registered[$date] = array($regp['ID'] => $regp);
        }

        // lista dni wolnych od pracy
        $excluded_dates = explode(', ', trim($config['excluded_days']));
        
        // przekazanie zmiennycyh do widoku
        $this->min_hour = $min_hour;
        $this->min_minute = $min_minute;
        $this->max_time = mktime($max_hour, $max_minute);
        $this->config = $config;
        $this->week_dates = $week_dates;
        $this->current_weeks_number = $current_week->format('W');
        $this->current_weeks_year = $current_week->format('Y');
        $this->next_weeks_number = $next_week->format('W');
        $this->next_weeks_year = $next_week->format('Y');
        $this->prev_weeks_number = $prev_week->format('W');
        $this->prev_weeks_year = $prev_week->format('Y');
        $this->start_time = $start_time;
        $this->stop_time = $stop_time;
        $this->reg_list = $registered;
        $this->excluded_dates = $excluded_dates;
        $this->dow = $dow;
        $this->week_number = $week_number;
        $this->year = $year;
        $this->reg_msg = $this->getUser()->hasFlash('info') ? $this->getUser()->getFlash('info') : $this->point->getOpis();
        
        // lista wykluczonych stanowisk
        $excluded_conf = UstawieniePeer::getProperties('registatrion_conf');
        $this->excluded_conf = array();
        foreach($excluded_conf as $key => $val)
        {
            list($point_id, $dow, $hour, $nr) = explode('_', $key);
            
            if(!array_key_exists($point_id, $this->excluded_conf))
            {
                $this->excluded_conf[$point_id] = array();
            }
            
            if(!array_key_exists($dow, $this->excluded_conf[$point_id]))
            {
                $this->excluded_conf[$point_id][$dow] = array();
            }
            
            if(!array_key_exists($hour, $this->excluded_conf[$point_id][$dow]))
            {
                $this->excluded_conf[$point_id][$dow][$hour] = 0;
            }
            
            $this->excluded_conf[$point_id][$dow][$hour] += 1;
        }

    }

    /**
     * akcja dodawania nowej rejestracji 
     * @author      Tomasz Glowinski, 2012
     * @param       sfRequest $request A request object
     */
    public function executeNew(sfWebRequest $request)
    {
        if($request->isMethod('post'))
        {
            $this->point_id = $request->getParameter('point_id');
            $point = RegistrationPointTblPeer::retrieveByPK($this->point_id);
            $this->forward404Unless($point);
            $this->point = $point;

            // odczyt zapisanych ustawien
            // $config = UstawieniePeer::getProperties('regp_config'); // v1
            $config = $point->toArray(BasePeer::TYPE_FIELDNAME); // v2

            // przechwycenie zgloszonych danych
            $date = str_replace('_', '-', $request->getParameter('date'));
            $time = urldecode($request->getParameter('time'));
            
            // walidacja danych
            if(empty($date) || empty($time))
            {
                $this->getUser()->setFlash('error', 'Niepoprawny parametr');
                $this->redirect('rejestracja/index');
            }

            $this->reg_msg = $config['pre_registration_message'];

            //dostepna ilosc czasu
            $start_time = date('Y-m-d H:i:s', strtotime($date.' '.$time));
            $choices = RegistrationTblPeer::getMaxTimeSpanByDate($start_time, $config);

            // obiekt formy    
            $this->form = new RegistrationTblForm(null, array('time_span_choices' => $choices, 'decorate' => true));
            $this->form->setDefault('date', $date.' '.$time);
            $this->form->setDefault('status', 0);
            $this->form->setDefault('tel', '');
            $this->form->setDefault('point_id', $this->point_id);
        }
        elseif($request->isMethod('get'))
        {
            // przechwycenie zgloszonych danych
            $params = $request->getParameter('params');

            if(empty($params))
            {
                $this->getUser()->setFlash('error', 'Niedozwolona operacja');
                $this->redirect('@regp_index');
            }
            else
                $params = unserialize($params);

            $this->point_id = $params['point_id'];
            $point = RegistrationPointTblPeer::retrieveByPK($this->point_id);
            $this->forward404Unless($point);
            $this->point = $point;

            // odczyt zapisanych ustawien
            // $config = UstawieniePeer::getProperties('regp_config'); // v1
            $config = $point->toArray(BasePeer::TYPE_FIELDNAME); // v2
            
            // odczytanie zapisanej konfiguracji
            $this->reg_msg = $config['pre_registration_message'];

            //dostepna ilosc czasu
            $start_time = date('Y-m-d H:i:s', strtotime($params['date']));
            $choices = RegistrationTblPeer::getMaxTimeSpanByDate($start_time, $config);

            // obiekt formy    
            $this->form = new RegistrationTblForm(null, array('time_span_choices' => $choices, 'decorate' => true));
            $this->form->bind($params);
        }
        else
        {
            $this->getUser()->setFlash('error', 'Niedozwolona operacja');
            $this->redirect('@regp_index');
        }
    }

    /**
     * akcja walidujaca dene i rejestrujaca osobe 
     * @param sfWebRequest $request 
     */
    public function executeRegister(sfWebRequest $request)
    {
        try
        {
            // walidacja zgloszenia
            if(!$request->isMethod('post'))
                throw new Exception('Niedozwolona operacja');

            // nowy obiekt formy
            $this->form = new RegistrationTblForm();

            // przechwycenie i oczyszczenie zgloszonych danych
            $params = $request->getParameter('registration_tbl');
            //devdump($params, '', 1);
            $safe_params = Functions::cleanValuesFromForm($this->form, $params);
            
            $point = RegistrationPointTblPeer::retrieveByPK($params['point_id']);
            if(empty($point))
                throw new Exception('Nie ma takiego wydziału ('.$params['point_id'].')');

            // laczenie danych z forma            
            unset($safe_params['id']);
            $this->form->bind($safe_params);

            // walidacja formy
            if(!($this->form->isValid()))
                throw new Exception('Wypełnij poprawnie wszystkie wymagane pola: '.PHP_EOL.strip_tags(implode(PHP_EOL, $this->form->getErrorsNice())));

            // weryfikacja dostepnosci terminu
            if(RegistrationTblPeer::checkAvailabilityByDate($params['date'].':00', $params['point_id']) == false)
                throw new Exception('Termin jest już zajęty');

            // wysylka maila z linkiem aktywacyjnym        
            // odczyt zapisanych ustawien
            // $config = UstawieniePeer::getProperties('regp_config'); // v1
            $config = $point->toArray(BasePeer::TYPE_FIELDNAME); // v2

            // zapisanie rejestracji
            $regp = $this->form->save();            
            $safe_params['obj'] = $regp;

            // wysylanie wiadomosci email na podany adres
            $mail_send_res = RegistrationTblPeer::sendVerificationMessage($safe_params, $config);
            if($mail_send_res !== true)
            {
                //$this->form->delete();
                $this->form->getObject()->delete();
                throw $mail_send_res;
            }

            // obsluga poszerzenia czasu rejestracji
            if(array_key_exists('time_span', $safe_params))
            {
                $time_span = $safe_params['time_span'];
                $start_time = date('Y-m-d H:i:s', strtotime($safe_params['date']));
                for($i = 1; $i < $time_span; $i++)
                {
                    $curr_time = date('Y-m-d H:i:s', strtotime($start_time) + $i * $config['interval'] * 60);
                    if(RegistrationTblPeer::checkAvailabilityByDate($curr_time) == false)
                        break;

                    $curr_form = $safe_params;
                    $curr_form['date'] = $curr_time;
                    $reg = new RegistrationTbl();
                    $reg->fromArray($curr_form, BasePeer::TYPE_FIELDNAME);
                    $reg->save();
                }
            }

            // komunikat o powodzeniu
            $this->getUser()->setFlash('notice', 'Zapisano rejestrację i wysłano wiadomość weryfikacyjną');
            $conv = array(
                '[STATUS]' => RegistrationTblPeer::getStatusNice($regp->getStatus()), 
                '[WYDZIAL]' => $point->getNazwa(),
                '[IMIE]' => $regp->getName(),
                '[NAZWISKO]' => $regp->getLastName(),
                '[DATA]' => date('j-m-Y', strtotime($regp->getDate())),
                '[GODZINA]' => date('H:i', strtotime($regp->getDate())),
                '[TEL]' => $regp->getTel(),
                '[EMAIL]' => $regp->getEmail(),
            );
            $message_content = str_replace(array_keys($conv), array_values($conv), $config['post_registration_message']);
            $this->getUser()->setFlash('info', $message_content);

            // przekierowanie na widok listy
            $this->redirect('@regp_show?point_id='.$point->getId().'&junk='.$point->getNazwa());
        }
        catch (sfStopException $e)
        {
            exit;
        }
        catch (Exception $e)
        {
            // komunikat o bledzie
            $this->getUser()->setFlash('error', htmlspecialchars($e->getMessage()));

            // przekierowanie na widok listy
            $this->redirect('rejestracja/new?params='.serialize($params));
        }
    }

    /**
     * akcja aktywujaca rejestracje
     * @param sfWebRequest $request 
     */
    public function executeActivate(sfWebRequest $request)
    {
        try
        {
            // przechwycenie danych
            $id = $request->getParameter('id');
            $key = $request->getParameter('key');

            // walidacja przeslanych danych
            if(empty($id) || empty($key))
                throw new Exception('Niepoprawny parametr');

            // odczytanie rekordu z bazy
            $regp = RegistrationTblPeer::retrieveByPK($id);
            if(empty($regp))
                throw new Exception('Nie ma takiego rekordu');
            
            // odczytanie zapisanej konfiguracji
            $point = $regp->getRegistrationPointTbl();
            // $config = UstawieniePeer::getProperties('regp_config'); // v1
            $config = $point->toArray(BasePeer::TYPE_FIELDNAME); // v2

            // weryfikacja klucza
            $expected_key = md5($regp->getEmail().$_SERVER['SERVER_NAME'].$regp->getName().substr($regp->getDate(), 0, 16).$regp->getLastName());
            if($expected_key != $key)
                throw new Exception('Niepoprawny kod aktywacyjny');

            // sprawdzam czy ta wstepna rejestracja jeszcze nie wygasla
            if(date('Y-m-d H:i:s', strtotime('+'.$config['verification_time'].' hours', strtotime($regp->getCreatedAt()))) < date('Y-m-d H:i:s'))
            {
                throw new Exception('Ważność tej rejestracji wygasła '.date('d-m-Y H:i:s', strtotime('+'.$config['verification_time'].' hours', strtotime($regp->getCreatedAt()))));
            }
            
            // weryfikacja dostepnosci terminu (w razie gdyby ktos inny juz potwierdzil rejestracje)
            $this_term_registrations = RegistrationTblPeer::getRegistratedPersonsByDate($regp->getDate(), $point->getId());
            $matches = false;
            foreach($this_term_registrations as $term_regp)
            {
                if($term_regp['ID'] == $regp->getId())
                {
                    $matches = true;
                    break;
                }
            }
            if(!RegistrationTblPeer::checkAvailabilityByDate($regp->getDate(), $point->getId()) && !$matches)
            {
                throw new Exception('Ten termin nie jest już dostępny');
            }

            // weryfikacja stanu rejestracji
            $status = $regp->getStatus();
            if($status & RegistrationTblPeer::$STATUS_CANCELED)
            {
                throw new Exception('Ta rezerwacja została wcześniej odwołana');
            }
            
            if($status & RegistrationTblPeer::$STATUS_CONFIRMED)
            {
                throw new Exception('Ta rezerwacja została już wcześniej potwierdzona');
            }
            
            // zapisanie statusu
            $regp->setStatus(RegistrationTblPeer::$STATUS_CONFIRMED);
            $regp->save();

            // obsluga rozszerzonego czasu rejestracji
//            $following_regs = $regp->getFollowingTimeSpan($config);
//            if(!empty($following_regs))
//                foreach($following_regs as $reg)
//                {
//                    $reg->setStatus(1);
//                    $reg->save();
//                }

            // aktualizacja statystyk
            StatisticsTblPeer::increaseUses('regp', $point->getId());
            
            // informacja o potwierdzeniu rejestracji na podany w konfiguracji email
            if(!empty($config['notice_email']) && !($status & 1))
            {
                $message_content = $regp->getName().' '.$regp->getLastName().' potwierdził(-a) rezerwację terminu '.substr($regp->getDate(), 0, 16).' w '.$point->getNazwa().'.';

                foreach(array_filter(explode(', ', trim($config['notice_email']))) as $email)
                {
                    $mailer = new ciMailer('Potwierdzono rejestrację', $email);
                    $mailer->attachContent($message_content, ciMailer::HTML);
                    $result = $mailer->send();
                    if($result !== true)
                        throw new Exception($result);
                }
            }

            //komunikat w przypadku powodzenia
            $this->redirect('rejestracja/confirmation?id='.$id.'&key='.$key);
        }
        catch (Exception $e)
        {
            $this->getUser()->setFlash('error', $e->getMessage());
        }

        $this->setTemplate('message');
        $this->point = $point;
        $this->point_id = $point->getId();
    }

    /**
     * akcja usuwajaca rejestracje i wyswietlajaca komunikat w widoku interanty
     * @param sfWebRequest $request 
     */
    public function executeDelete(sfWebRequest $request)
    {
        try
        {
            // przechwycenie danych
            $id = $request->getParameter('id');
            $key = $request->getParameter('key');

            // walidacja przeslanych danych
            if(empty($id) || empty($key))
                throw new Exception('Niepoprawny parametr');

            // odczytanie rekordu z bazy
            $regp = RegistrationTblPeer::retrieveByPK($id);
            if(empty($regp))
                throw new Exception('Nie ma takiego rekordu');

            // odczytanie zapisanej konfiguracji
            $point = $regp->getRegistrationPointTbl();
            // $config = UstawieniePeer::getProperties('regp_config'); // v1
            $config = $point->toArray(BasePeer::TYPE_FIELDNAME); // v2

            // weryfikacja klucza
            $expected_key = md5($regp->getEmail().$_SERVER['SERVER_NAME'].$regp->getName().substr($regp->getDate(), 0, 16).$regp->getLastName());
            if($expected_key != $key)
                throw new Exception('Niepoprawny kod aktywacyjny');

            $status = $regp->getStatus();
            
            // aktualizacja statusu
            $regp->setStatus(2);
            $regp->save();

            if($status == RegistrationTblPeer::$STATUS_CONFIRMED)
            {                
                // aktualizacja statystyk
                try
                {
                    StatisticsTblPeer::decreaseUses('regp', $point->getId());            
                }
                catch(Exception $e)
                {
                    $this->getUser()->setFlash('error', 'Nie można zaktualizować statystyk.');
                }
            }
            
            // informacja o usunieciu rejestracji na podany w konfiguracji email
            if(!empty($config['notice_email']) && ($status & 1) && !($status & 2))
            {
                $message_content = $regp->getName().' '.$regp->getLastName().' odwołał(-a) rezerwację terminu '.substr($regp->getDate(), 0, 16).' w '.$point_>getNazwa().'.';

                foreach(array_filter(explode(', ', trim($config['notice_email']))) as $email)
                {
                    $mailer = new ciMailer('Odwołano rejestrację', $email);
                    $mailer->attachContent($message_content, ciMailer::HTML);
                    $result = $mailer->send();
                    if($result !== true)
                        throw new Exception($result);
                }
            }

            //komunikat w przypadku powodzenia
            $this->getUser()->setFlash('notice', 'Rezerwacja została odwołana');
        }
        catch (Exception $e)
        {
            $this->getUser()->setFlash('error', $e->getMessage());
        }

        $this->setTemplate('message');
        $this->point = $point;
        $this->point_id = $point->getId();
    }

    /**
     * akcja wyswietlajaca potwierdzenie
     * @param sfWebRequest $request 
     */
    public function executeConfirmation(sfWebRequest $request)
    {
        try
        {
            // przechwycenie danych
            $id = $request->getParameter('id');
            $key = $request->getParameter('key');

            // walidacja przeslanych danych
            if(empty($id) || empty($key))
                throw new Exception('Niepoprawny parametr');

            // odczytanie rekordu z bazy
            $regp = RegistrationTblPeer::retrieveByPK($id);
            if(empty($regp))
                throw new Exception('Nie ma takiego rekordu');

            // odczytanie zapisanej konfiguracji
            $point = $regp->getRegistrationPointTbl();
            // $config = UstawieniePeer::getProperties('regp_config'); // v1
            $config = $point->toArray(BasePeer::TYPE_FIELDNAME); // v2

            // weryfikacja klucza
            $expected_key = md5($regp->getEmail().$_SERVER['SERVER_NAME'].$regp->getName().substr($regp->getDate(), 0, 16).$regp->getLastName());
            if($expected_key != $key)
                throw new Exception('Niepoprawny kod aktywacyjny');

            $status = $regp->getStatus();

            //komunikat w zaleznosci od statusu
            if($status & 8)
            {
                $this->getUser()->setFlash('notice', 'Rezerwacja WYKLUCZONA');
            }
            elseif($status & 2)
            {
                $this->getUser()->setFlash('notice', 'Rezerwacja ODWOŁANA');
            }
            elseif($status & 01)
            {
                $conv = array(
                    '[STATUS]' => RegistrationTblPeer::getStatusNice($regp->getStatus()), 
                    '[WYDZIAL]' => $regp->getRegistrationpointTbl()->getNazwa(),
                    '[IMIE]' => $regp->getName(),
                    '[NAZWISKO]' => $regp->getLastName(),
                    '[DATA]' => date('j-m-Y', strtotime($regp->getDate())),
                    '[GODZINA]' => date('H:i', strtotime($regp->getDate())),
                    '[TEL]' => $regp->getTel(),
                    '[EMAIL]' => $regp->getEmail(),
                );
                $this->msg = str_replace(array_keys($conv), array_values($conv), $config['post_registration_message']);
                $this->getUser()->setFlash('notice', 'Rejestracja została potwierdzona');
            }
            else
            {
                $this->getUser()->setFlash('notice', 'Rezerwacja NIEPOTWIERDZONA');
            }
        }
        catch (Exception $e)
        {
            $this->getUser()->setFlash('error', $e->getMessage());
        }

        $this->setTemplate('message');
        $this->point = $point;
        $this->point_id = $point->getId();
    }

}
?>