<?php

/**
 * Klasa rejestracji osob.
 * @author  Tomasz Glowinski, 2012
 */
class RegistrationTblPeer extends BaseRegistrationTblPeer
{
    public static $STATUS_UNCONFIRMED = 0;
    public static $STATUS_CONFIRMED = 1;
    public static $STATUS_CANCELED = 2;
    public static $STATUS_BLOCKED = 8;
    
    /**
     * metoda zwracajaca zrozumialy dla smiertelnika status rejestracji
     * @param       int $status cyfrowy zapis stanu tej rejestracji
     * @return      string opis stanu rejestracji
     * @author      Tomasz Glowinski, 2016
     */
    public static function getStatusNice($status)
    {
        if($status & self::$STATUS_BLOCKED)
            return 'Wyłączony dostęp';
        elseif($status & self::$STATUS_CONFIRMED)
            return 'Potwierdzono';
        elseif($status & self::$STATUS_CANCELED)
            return 'Odwołano';
        else
            return 'Niepotwierdzono';
    }
    
    /**
     * metoda zwracajaca pelna liste dokonanych rejestracji na wskazany tydzien
     * @author  Tomasz Glowinski, 2012
     * @param   Integer $week_number numer tygodnia w roku
     * @param   $point_id id wydzialu
     * @param   String $year rok w formacie YYYY
     * @return  Array pelna lista dokonanych rejestracji na dany dzien
     */
    public static function getRegistratedPersonsByWeek($week, $point_id, $year = null)
    {
        if(empty($year))
            $year = date('Y');

        $week_start = new DateTime();
        $week_start->setISODate($year, $week);

        $week_stop = new DateTime();
        $week_stop->setISODate($year, $week + 1);

        // odczyt zapisanych ustawien
        $point = RegistrationPointTblPeer::retrieveByPK($point_id);        
        // $config = UstawieniePeer::getProperties('regp_config'); // v1
        $config = $point->toArray(BasePeer::TYPE_FIELDNAME); // v2

        $c = new CriteriaExt();
        $c->add(RegistrationTblPeer::DATE, $week_start->format('Y-m-d').' 00:00:00', Criteria::GREATER_EQUAL);
        $c->addAnd(RegistrationTblPeer::DATE, $week_stop->format('Y-m-d').' 00:00:00', Criteria::LESS_THAN);
        $c->add(RegistrationTblPeer::POINT_ID, $point_id);
        $c0 = $c->getNewCriterion(RegistrationTblPeer::STATUS, 2, Criteria::NOT_EQUAL);
        $c1 = $c->getNewCriterion(RegistrationTblPeer::STATUS, 0, Criteria::NOT_EQUAL);
        $c2 = $c->getNewCriterion(RegistrationTblPeer::STATUS, 0);
        $c3 = $c->getNewCriterion(RegistrationTblPeer::CREATED_AT, date('Y-m-d H:i:s', strtotime("-".$config['verification_time'].' hours', strtotime("now"))), Criteria::GREATER_EQUAL);
        $c2->addAnd($c3);
        $c1->addOr($c2);
        $c0->addAnd($c1);
        $c->add($c0);
        $c->clearSelectColumns();
        $c->addSelectColumn(RegistrationTblPeer::DATE)
            ->addSelectColumn(RegistrationTblPeer::ID)
            ->addSelectColumn(RegistrationTblPeer::NAME)
            ->addSelectColumn(RegistrationTblPeer::LAST_NAME)
            ->addSelectColumn(RegistrationTblPeer::TEL)
            ->addSelectColumn(RegistrationTblPeer::EMAIL)
            ->addSelectColumn(RegistrationTblPeer::CREATED_AT)
            ->addSelectColumn(RegistrationTblPeer::UPDATED_AT)
            ->addSelectColumn(RegistrationTblPeer::STATUS);

        return RegistrationTblPeer::doSelectStmt($c)->fetchAll(PDO::FETCH_ASSOC);
    }

    /**
     * metoda zwracajaca pelna liste dokonanych rejestracji na wskazany dzien / godzine
     * @author  Tomasz Glowinski, 2012
     * @param   string $date data w formacie YYYY-MM-DD HH:MM:SS
     * @param   $point_id id wydzialu
     * @return  Array pelna lista dokonanych rejestracji na dany dzien
     */
    public static function getRegistratedPersonsByDate($date, $point_id)
    {
        // odczyt zapisanych ustawien
        $point = RegistrationPointTblPeer::retrieveByPK($point_id);        
        // $config = UstawieniePeer::getProperties('regp_config'); // v1
        $config = $point->toArray(BasePeer::TYPE_FIELDNAME); // v2

        $c = new CriteriaExt();
        $c->add(RegistrationTblPeer::DATE, $date);
        $c->add(RegistrationTblPeer::POINT_ID, $point_id);
        $c0 = $c->getNewCriterion(RegistrationTblPeer::STATUS, 2, Criteria::NOT_EQUAL);
        $c1 = $c->getNewCriterion(RegistrationTblPeer::STATUS, 0, Criteria::NOT_EQUAL);
        $c2 = $c->getNewCriterion(RegistrationTblPeer::STATUS, 0);
        $c3 = $c->getNewCriterion(RegistrationTblPeer::CREATED_AT, date('Y-m-d H:i:s', strtotime("-".$config['verification_time'].' hours', strtotime("now"))), Criteria::GREATER_EQUAL);
        $c2->addAnd($c3);
        $c1->addOr($c2);
        $c0->addAnd($c1);
        $c->add($c0);
        $c->clearSelectColumns();
        $c->addSelectColumn(RegistrationTblPeer::DATE)
            ->addSelectColumn(RegistrationTblPeer::ID)
            ->addSelectColumn(RegistrationTblPeer::CREATED_AT)
            ->addSelectColumn(RegistrationTblPeer::UPDATED_AT)
            ->addSelectColumn(RegistrationTblPeer::STATUS);

        return RegistrationTblPeer::doSelectStmt($c)->fetchAll(PDO::FETCH_ASSOC);
    }

    /**
     * metoda zwracajaca skrocona liste dokonanych rejestracji na wskazany tydzien
     * @author  Tomasz Glowinski, 2012
     * @param   Integer $week_number numer tygodnia w roku
     * @param   $point_id id wydzialu
     * @param   String $year rok w formacie YYYY
     * @return  Array skroconalista dokonanych rejestracji na dany dzien
     */
    public static function getShortRegistratedPersonsByWeek($week, $point_id, $year = null)
    {
        if(empty($year))
            $year = date('Y');

        $week_start = new DateTime();
        $week_start->setISODate($year, $week);

        $week_stop = new DateTime();
        $week_stop->setISODate($year, $week + 1);
        
        // odczyt zapisanych ustawien
        $point = RegistrationPointTblPeer::retrieveByPK($point_id);        
        // $config = UstawieniePeer::getProperties('regp_config'); // v1
        $config = $point->toArray(BasePeer::TYPE_FIELDNAME); // v2
        
        $date_to = min(date('Y-m-d', strtotime('+'.($config['forth_browse_days']+1).' days')), $week_stop->format('Y-m-d')).' 00:00:00';
        $date_from = max(date('Y-m-d', strtotime('-'.$config['back_browse_days'].' days')), $week_start->format('Y-m-d')).' 00:00:00';
        
        $c = new CriteriaExt();
        $c->add(RegistrationTblPeer::DATE, $date_from, Criteria::GREATER_EQUAL);
        $c->addAnd(RegistrationTblPeer::DATE, $date_to, Criteria::LESS_THAN);
        $c->add(RegistrationTblPeer::POINT_ID, $point_id);
        $c0 = $c->getNewCriterion(RegistrationTblPeer::STATUS, 2, Criteria::NOT_EQUAL);
        $c1 = $c->getNewCriterion(RegistrationTblPeer::STATUS, 0, Criteria::NOT_EQUAL);
        $c2 = $c->getNewCriterion(RegistrationTblPeer::STATUS, 0);
        $c3 = $c->getNewCriterion(RegistrationTblPeer::DATE, date('Y-m-d H:i:s', strtotime("-".$config['verification_time'].' hours')), Criteria::GREATER_EQUAL);
        $c2->addAnd($c3);
        $c1->addOr($c2);
        $c0->addAnd($c1);
        $c->add($c0);
        $c->clearSelectColumns();
        $c->addSelectColumn(RegistrationTblPeer::DATE)
            ->addSelectColumn(RegistrationTblPeer::ID)
            ->addSelectColumn(RegistrationTblPeer::STATUS);
        
        return RegistrationTblPeer::doSelectStmt($c)->fetchAll(PDO::FETCH_ASSOC);
    }

    /**
     * metoda informujaca czy termin ma jeszcze wolne miejsca
     * @author  Tomasz Glowinski, 2012
     * @param   String $datetime dzien w formacie YYYY-MM-DD HH:MM:00
     * @param   $point_id id wydzialu
     * @param   &$ids array do jakiej zostana przypisane id rekordow
     * @return  Boolean true jesli termin jest wolny, false w p.p.
     */
    public static function checkAvailabilityByDate($datetime, $point_id, $ids = null)
    {
        if($datetime < date('Y-m-d H:i:s') && !sfContext::getInstance()->getUser()->isSuperAdmin())
            return false;
        
        $point = RegistrationPointTblPeer::retrieveByPK($point_id);
        if(empty($point))
        {
            throw new Exception('Nie ma takiego wydzialu ('.$point_id.')');
        }

        //$excluded_days = explode(', ', $point->getExcludedDays());
        
        $valid_date = date('Y-m-d H:i:s', strtotime("-".$point->getVerificationTime()." hours"));
        
        $c = new CriteriaExt();
        $c->add(RegistrationTblPeer::DATE, $datetime);
        $c->add(RegistrationTblPeer::POINT_ID, $point_id);
        $c0 = $c->getNewCriterion(RegistrationTblPeer::STATUS, 2, Criteria::NOT_EQUAL);
        $c1 = $c->getNewCriterion(RegistrationTblPeer::STATUS, 0, Criteria::NOT_EQUAL);
        $c2 = $c->getNewCriterion(RegistrationTblPeer::STATUS, 0);
        $c3 = $c->getNewCriterion(RegistrationTblPeer::CREATED_AT, $valid_date, Criteria::GREATER_EQUAL);
        $c2->addAnd($c3);
        $c1->addOr($c2);
        $c0->addAnd($c1);
        $c->add($c0);
        $rs = RegistrationTblPeer::doSelect($c);

        if(is_array($ids))
        {
            foreach($rs as $rec)
            {
                $ids[] = $rec;
            }
        }
        
        if(count($rs) > 0)
        {
            return $point->getParallelPoints() - count($rs);
        }
        else
            return $point->getParallelPoints();
    }

    /**
     * metoda informujaca czy termin ma jeszcze wolne miejsca
     * @author  Tomasz Glowinski, 2012
     * @param   String $datetime dzien w formacie YYYY-MM-DD HH:MM:00
     * @param   $point_id id wydzialu
     * @param   &$ids array do jakiej zostana przypisane id rekordow
     * @return  Boolean true jesli termin jest wolny, false w p.p.
     */
    public static function checkDuplicationByDate($datetime, $point_id, $email, $ids = null)
    {
        if($datetime < date('Y-m-d H:i:s') && !sfContext::getInstance()->getUser()->isSuperAdmin())
            return false;
        
        $point = RegistrationPointTblPeer::retrieveByPK($point_id);
        if(empty($point))
        {
            throw new Exception('Nie ma takiego wydzialu ('.$point_id.')');
        }

        //$excluded_days = explode(', ', $point->getExcludedDays());
        
        $valid_date = date('Y-m-d H:i:s', strtotime("-".$point->getVerificationTime()." hours"));
        $od = date('Y-m-d 00:00:00', strtotime($datetime));
        $do = date('Y-m-d 23:59:59', strtotime($datetime));
        
        $c = new CriteriaExt();
        $c->add(RegistrationTblPeer::EMAIL, $email);
        $c->add(RegistrationTblPeer::POINT_ID, $point_id);
        $c0 = $c->getNewCriterion(RegistrationTblPeer::DATE, $od, Criteria::GREATER_EQUAL);
        $c1 = $c->getNewCriterion(RegistrationTblPeer::DATE, $do, Criteria::LESS_EQUAL);
        $c0->addAnd($c1);
        $c->add($c0);
        $rs = RegistrationTblPeer::doSelect($c);

        if(count($rs) > 0)
        {
            return false;
        }
        else
            return true;
    }

    /**
     * metoda odczytujaca rekord rejestracji z bazy i wysylajaca maila do uzytkownika
     * @author      Tomasz Glowinski, 2012
     * @param       type $params 
     * @return      Boolean true w przypadku powodzenia, Exception w p.p.
     */
    public static function sendVerificationMessage($params, $config)
    {
        $regp = $params['obj'];
        if(empty($regp))
            return new Exception('Nie ma takiego wpisu');
        
        $key = md5($params['email'].$_SERVER['SERVER_NAME'].$params['name'].$params['date'].$params['last_name']);
        $link_activate = '<a href="http://'.$_SERVER['SERVER_NAME'].'/rejestracja/activate.html?id='.$regp->getId().'&key='.$key.'">Aktywuj</a>';
        $link_delete = '<a href="http://'.$_SERVER['SERVER_NAME'].'/rejestracja/delete.html?id='.$regp->getId().'&key='.$key.'">Usuń z rejestracji</a>';
        $desc_state = self::getStatusNice($regp->getStatus());        
        $conv = array(
            '[AKTYWACJA]' => $link_activate,
            '[USUN]' => $link_delete,
            '[STATUS]' => $desc_state, 
            '[WYDZIAL]' => $regp->getRegistrationpointTbl()->getNazwa(),
            '[IMIE]' => $regp->getName(),
            '[NAZWISKO]' => $regp->getLastName(),
            '[DATA]' => date('j-m-Y', strtotime($regp->getDate())),
            '[GODZINA]' => date('H:i', strtotime($regp->getDate())),
            '[TEL]' => $regp->getTel(),
            '[EMAIL]' => $regp->getEmail(),
        );
        $message_content = $config['verification_message'];
        $message_content = str_replace(array_keys($conv), array_values($conv), $message_content);

        $mailer = new ciMailer('Rejestracja', $params['email']);
        $mailer->attachContent($message_content, ciMailer::HTML);
        $result = $mailer->send();
        if($result !== true)
            throw new Exception($result);

        //if (mail($params['name'] . ' ' . $params['last_name'] . ' <' . $params['email'] . '>', 'REJESTRACJA', $message_content, $message_headers) === false)
        //    return new Exception('Nie udało się wysłać wiadomości aktywacyjnej');
        
        // komunikat w przypadku powodzenia
        return true;
    }

    /**
     * metoda sprawdzajaca ile czasu mozna zarezerwowac
     * @param   String $datetime dzien w formacie YYYY-MM-DD HH:MM:00
     * @param   $interval interwal czasowy (krok sprawdzenia)
     * @return  array lista wolnych terminow w postaci opcji do select-a
     * @author  Tomasz Glowinski, 2015
     */
    public static function getMaxTimeSpanByDate($datetime, $config)
    {
        if($datetime < date('Y-m-d H:i:s'))
            return null;
        
        if(!array_key_exists('interval', $config))
            return null;
        
        $interval = $config['interval'];
        $stop_time = date("Y-m-d", strtotime($datetime)).' '.$config[strtolower(date('D', strtotime($datetime))).'_stop'].':00';
        
        $choices = array();
        $key = 1;
        $slot = $datetime;
        while(self::checkAvailabilityByDate($slot, $config['id']) && $slot < $stop_time)
        {
            $choices[$key] = $key*$interval;
            $slot = date('Y-m-d H:i:s', strtotime($slot) + $interval*60);
            $key++;
        }
        
        return $choices;
    }

}